package java.time.chrono;

import sun.util.calendar.CalendarSystem;
import java.io.InvalidObjectException;
import java.io.ObjectInputStream;
import java.time.temporal.ChronoUnit;
import java.time.temporal.TemporalAdjusters;
import java.time.format.ResolverStyle;
import java.time.temporal.TemporalField;
import java.util.Map;
import java.util.Calendar;
import java.time.temporal.UnsupportedTemporalTypeException;
import java.time.temporal.ValueRange;
import java.time.temporal.ChronoField;
import java.util.Arrays;
import java.util.List;
import java.time.DateTimeException;
import java.time.Instant;
import java.time.temporal.TemporalAccessor;
import java.time.ZoneId;
import java.time.Clock;
import java.time.LocalDate;
import java.util.Locale;
import sun.util.calendar.LocalGregorianCalendar;
import java.io.Serializable;

public final class JapaneseChronology extends AbstractChronology implements Serializable
{
    static final LocalGregorianCalendar JCAL;
    static final Locale LOCALE;
    public static final JapaneseChronology INSTANCE;
    private static final long serialVersionUID = 459996390165777884L;
    
    private JapaneseChronology() {
    }
    
    @Override
    public String getId() {
        return "Japanese";
    }
    
    @Override
    public String getCalendarType() {
        return "japanese";
    }
    
    @Override
    public JapaneseDate date(Era era, int n, int n2, int n3) {
        if (!(era instanceof JapaneseEra)) {
            throw new ClassCastException("Era must be JapaneseEra");
        }
        return JapaneseDate.of((JapaneseEra)era, n, n2, n3);
    }
    
    @Override
    public JapaneseDate date(int year, int month, int dayOfMonth) {
        return new JapaneseDate(LocalDate.of(year, month, dayOfMonth));
    }
    
    @Override
    public JapaneseDate dateYearDay(Era era, int n, int n2) {
        return JapaneseDate.ofYearDay((JapaneseEra)era, n, n2);
    }
    
    @Override
    public JapaneseDate dateYearDay(int year, int dayOfYear) {
        return new JapaneseDate(LocalDate.ofYearDay(year, dayOfYear));
    }
    
    @Override
    public JapaneseDate dateEpochDay(long epochDay) {
        return new JapaneseDate(LocalDate.ofEpochDay(epochDay));
    }
    
    @Override
    public JapaneseDate dateNow() {
        return this.dateNow(Clock.systemDefaultZone());
    }
    
    @Override
    public JapaneseDate dateNow(ZoneId zone) {
        return this.dateNow(Clock.system(zone));
    }
    
    @Override
    public JapaneseDate dateNow(Clock clock) {
        return this.date(LocalDate.now(clock));
    }
    
    @Override
    public JapaneseDate date(TemporalAccessor temporal) {
        if ((temporal instanceof JapaneseDate)) {
            return (JapaneseDate)temporal;
        }
        return new JapaneseDate(LocalDate.from(temporal));
    }
    
    @Override
    public ChronoLocalDateTime<JapaneseDate> localDateTime(TemporalAccessor temporalAccessor) {
        return super.localDateTime(temporalAccessor);
    }
    
    @Override
    public ChronoZonedDateTime<JapaneseDate> zonedDateTime(TemporalAccessor temporalAccessor) {
        return super.zonedDateTime(temporalAccessor);
    }
    
    @Override
    public ChronoZonedDateTime<JapaneseDate> zonedDateTime(Instant instant, ZoneId zoneId) {
        return super.zonedDateTime(instant, zoneId);
    }
    
    @Override
    public boolean isLeapYear(long n) {
        return IsoChronology.INSTANCE.isLeapYear(n);
    }
    
    @Override
    public int prolepticYear(Era era, int year) {
        if (!(era instanceof JapaneseEra)) {
            throw new ClassCastException("Era must be JapaneseEra");
        }
        JapaneseEra japaneseEra = (JapaneseEra)era;
        int n = japaneseEra.getPrivateEra().getSinceDate().getYear() + year - 1;
        if (year == 1) {
            return n;
        }
        if (n >= -999999999 && n <= 999999999) {
            LocalGregorianCalendar.Date calendarDate = JapaneseChronology.JCAL.newCalendarDate(null);
            calendarDate.setEra(japaneseEra.getPrivateEra()).setDate(year, 1, 1);
            if (JapaneseChronology.JCAL.validate(calendarDate)) {
                return n;
            }
        }
        throw new DateTimeException("Invalid yearOfEra value");
    }
    
    @Override
    public JapaneseEra eraOf(int n) {
        return JapaneseEra.of(n);
    }
    
    @Override
    public List<Era> eras() {
        return Arrays.asList(JapaneseEra.values());
    }
    
    JapaneseEra getCurrentEra() {
        JapaneseEra[] values = JapaneseEra.values();
        return values[values.length - 1];
    }
    
    @Override
    public ValueRange range(ChronoField obj) {
        switch (obj) {
            case ALIGNED_DAY_OF_WEEK_IN_MONTH:
            case ALIGNED_DAY_OF_WEEK_IN_YEAR:
            case ALIGNED_WEEK_OF_MONTH:
            case ALIGNED_WEEK_OF_YEAR: {
                throw new UnsupportedTemporalTypeException("Unsupported field: " + obj);
            }
            case YEAR_OF_ERA: {
                Calendar instance = Calendar.getInstance(JapaneseChronology.LOCALE);
                return ValueRange.of(1L, (long)instance.getGreatestMinimum(1), (long)(instance.getLeastMaximum(1) + 1), (long)(999999999 - this.getCurrentEra().getPrivateEra().getSinceDate().getYear()));
            }
            case DAY_OF_YEAR: {
                Calendar instance2 = Calendar.getInstance(JapaneseChronology.LOCALE);
                int n = 6;
                return ValueRange.of((long)instance2.getMinimum(n), (long)instance2.getGreatestMinimum(n), (long)instance2.getLeastMaximum(n), (long)instance2.getMaximum(n));
            }
            case YEAR: {
                return ValueRange.of((long)JapaneseDate.MEIJI_6_ISODATE.getYear(), 999999999L);
            }
            case ERA: {
                return ValueRange.of((long)JapaneseEra.MEIJI.getValue(), (long)this.getCurrentEra().getValue());
            }
            default: {
                return obj.range();
            }
        }
    }
    
    @Override
    public JapaneseDate resolveDate(Map<TemporalField, Long> map, ResolverStyle resolverStyle) {
        return (JapaneseDate)super.resolveDate(map, resolverStyle);
    }
    
    @Override
    ChronoLocalDate resolveYearOfEra(Map<TemporalField, Long> map, ResolverStyle resolverStyle) {
        Long n = (Long)map.get(ChronoField.ERA);
        JapaneseEra era = null;
        if (n != null) {
            era = this.eraOf(this.range(ChronoField.ERA).checkValidIntValue((long)n, ChronoField.ERA));
        }
        Long n2 = (Long)map.get(ChronoField.YEAR_OF_ERA);
        int checkValidIntValue = 0;
        if (n2 != null) {
            checkValidIntValue = this.range(ChronoField.YEAR_OF_ERA).checkValidIntValue((long)n2, ChronoField.YEAR_OF_ERA);
        }
        if (era == null && n2 != null && !map.containsKey(ChronoField.YEAR) && resolverStyle != ResolverStyle.STRICT) {
            era = JapaneseEra.values()[JapaneseEra.values().length - 1];
        }
        if (n2 != null && era != null) {
            if (map.containsKey(ChronoField.MONTH_OF_YEAR) && map.containsKey(ChronoField.DAY_OF_MONTH)) {
                return this.resolveYMD(era, checkValidIntValue, map, resolverStyle);
            }
            if (map.containsKey(ChronoField.DAY_OF_YEAR)) {
                return this.resolveYD(era, checkValidIntValue, map, resolverStyle);
            }
        }
        return null;
    }
    
    private int prolepticYearLenient(JapaneseEra japaneseEra, int n) {
        return japaneseEra.getPrivateEra().getSinceDate().getYear() + n - 1;
    }
    
    private ChronoLocalDate resolveYMD(JapaneseEra obj, int n, Map<TemporalField, Long> map, ResolverStyle resolverStyle) {
        map.remove(ChronoField.ERA);
        map.remove(ChronoField.YEAR_OF_ERA);
        if (resolverStyle == ResolverStyle.LENIENT) {
            return this.date(this.prolepticYearLenient(obj, n), 1, 1).plus(Math.subtractExact((long)map.remove(ChronoField.MONTH_OF_YEAR), 1L), ChronoUnit.MONTHS).plus(Math.subtractExact((long)map.remove(ChronoField.DAY_OF_MONTH), 1L), ChronoUnit.DAYS);
        }
        int checkValidIntValue = this.range(ChronoField.MONTH_OF_YEAR).checkValidIntValue((long)map.remove(ChronoField.MONTH_OF_YEAR), ChronoField.MONTH_OF_YEAR);
        int checkValidIntValue2 = this.range(ChronoField.DAY_OF_MONTH).checkValidIntValue((long)map.remove(ChronoField.DAY_OF_MONTH), ChronoField.DAY_OF_MONTH);
        if (resolverStyle == ResolverStyle.SMART) {
            if (n < 1) {
                throw new DateTimeException("Invalid YearOfEra: " + n);
            }
            int prolepticYearLenient = this.prolepticYearLenient(obj, n);
            JapaneseDate japaneseDate;
            try {
                japaneseDate = this.date(prolepticYearLenient, checkValidIntValue, checkValidIntValue2);
            }
            catch (DateTimeException ex) {
                japaneseDate = this.date(prolepticYearLenient, checkValidIntValue, 1).with(TemporalAdjusters.lastDayOfMonth());
            }
            if (japaneseDate.getEra() != obj && japaneseDate.get(ChronoField.YEAR_OF_ERA) > 1 && n > 1) {
                throw new DateTimeException("Invalid YearOfEra for Era: " + obj + " " + n);
            }
            return japaneseDate;
        }
        else {
            return this.date(obj, n, checkValidIntValue, checkValidIntValue2);
        }
    }
    
    private ChronoLocalDate resolveYD(JapaneseEra japaneseEra, int n, Map<TemporalField, Long> map, ResolverStyle resolverStyle) {
        map.remove(ChronoField.ERA);
        map.remove(ChronoField.YEAR_OF_ERA);
        if (resolverStyle == ResolverStyle.LENIENT) {
            return this.dateYearDay(this.prolepticYearLenient(japaneseEra, n), 1).plus(Math.subtractExact((long)map.remove(ChronoField.DAY_OF_YEAR), 1L), ChronoUnit.DAYS);
        }
        return this.dateYearDay(japaneseEra, n, this.range(ChronoField.DAY_OF_YEAR).checkValidIntValue((long)map.remove(ChronoField.DAY_OF_YEAR), ChronoField.DAY_OF_YEAR));
    }
    
    @Override
    Object writeReplace() {
        return super.writeReplace();
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws InvalidObjectException {
        throw new InvalidObjectException("Deserialization via serialization delegate");
    }
    
    static {
        JCAL = (LocalGregorianCalendar)CalendarSystem.forName("japanese");
        LOCALE = Locale.forLanguageTag("ja-JP-u-ca-japanese");
        INSTANCE = new JapaneseChronology();
    }
}
